package com.tomwheeler.stocktrader.ui.marketnews;

import java.awt.BorderLayout;
import java.awt.Rectangle;

import javax.swing.*;

import com.tomwheeler.stocktrader.api.Game;
import com.tomwheeler.stocktrader.api.events.*;

/**
 * <p>
 * This class provides a component for displaying market news in the StockTrader
 * game.  It provides the main part of the UI rendered by the MarketNewsTopComponent.
 * There's nothing specific to NetBeans in this class.
 * </p>
 * 
 * @author Tom Wheeler
 */
public class MarketNewsComponent extends JPanel {

    /**  number of stories displayed without scrolling */
    protected static int NUM_VISIBLE_STORIES = 8;

    private JList newsItemList;
    private DefaultListModel newsModel;
    private Game game;
    
    private NewsStoryListener newsListener;
    private GameListener currentGameListener;
    
    public MarketNewsComponent(final Game game) {
        super(new BorderLayout());

        this.game = game;
        
        newsModel = new DefaultListModel();
        newsItemList = new JList(newsModel);
        newsItemList.setVisibleRowCount(NUM_VISIBLE_STORIES);
        JScrollPane scroller = new JScrollPane(newsItemList,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        add(scroller, BorderLayout.CENTER);
        
        newsListener = new NewsListener();
        currentGameListener = new CurrentGameListener();
        
        game.addGameListener(currentGameListener);
    }
    
    // so we can update when new stories are published
    private class NewsListener implements NewsStoryListener {

        public void newsStoryPublished(NewsStoryEvent event) {
            newsModel.addElement(event.getStory().getStoryText());
            
            // ensures that the latest stories (at the bottom) are visible
            int lastItemIndex = newsModel.getSize() -1;
            if (lastItemIndex >= 0) {
                Rectangle end = newsItemList.getCellBounds(lastItemIndex, lastItemIndex);
                if (end != null) {
                    newsItemList.scrollRectToVisible(end);    
                }                
            }
        }
    }
    
    // keeps track of game start/stop, so we can reinitialize
    private class CurrentGameListener implements GameListener {

        public void gameStarted(GameChangeEvent event) {
            game.getDataProvider().addNewsStoryListener(newsListener);
        }

        public void gameEnded(GameChangeEvent event) {
            newsModel.clear();
        }
        
        public void dayChanged(GameChangeEvent event) {}
    }
}
