/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2005 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package com.tomwheeler.stocktrader.ui.portfolio;

import java.awt.Image;

import javax.swing.ImageIcon;

import org.openide.util.*;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * <p>
 * This class provides a system action for displaying the PortfolioTopComponent, 
 * which shows the stocks held by the player in the stock trader game.
 * </p> 
 * 
 * @author Tom Wheeler
 */
public class DisplayPortfolioAction extends CallableSystemAction {

    /* (non-Javadoc)
     * @see org.openide.util.SharedClassObject#initialize()
     */
    protected void initialize() {
        super.initialize();
        
        String path = "com/tomwheeler/stocktrader/ui/resources/portfolio-icon.gif";
        Image img = Utilities.loadImage(path);
        ImageIcon icon = new ImageIcon(Utilities.loadImage(path));
        setIcon(icon);
    }

    /* (non-Javadoc)
     * @see org.openide.util.actions.CallableSystemAction#performAction()
     */
    public void performAction() {
        TopComponent tc = PortfolioTopComponent.getInstance();
        if (!tc.isOpened()) {
            tc.open();
        }
        tc.requestActive();
    }

    /* (non-Javadoc)
     * @see org.openide.util.actions.SystemAction#getName()
     */
    public String getName() {
        return NbBundle.getMessage(DisplayPortfolioAction.class,
                "LBL_DisplayPortfolioAction_DisplayName");
    }

    /* (non-Javadoc)
     * @see org.openide.util.HelpCtx$Provider#getHelpCtx()
     */
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    /* (non-Javadoc)
     * @see org.openide.util.actions.CallableSystemAction#asynchronous()
     */
    protected boolean asynchronous() {
        return false;
    }

    /* (non-Javadoc)
     * @see javax.swing.Action#isEnabled()
     */
    public boolean isEnabled() {
        return (! PortfolioTopComponent.getInstance().isOpened());
    }
}
